use std::sync::Arc;

use sea_orm::{ConnectOptions, Database};
use tokio::signal;
use tracing_subscriber::{EnvFilter, fmt, layer::SubscriberExt, util::SubscriberInitExt};
use tracing_subscriber::fmt::time::LocalTime;

use axum_with_seaorm::{config::AppConfig, router, state::AppState};

fn init_trace_logger() {
    // 设置日志级别 根据RUST_LOG
    let env_filter = EnvFilter::try_from_default_env().unwrap_or_else(|_| EnvFilter::new("info"));

    let timer = LocalTime::rfc_3339();
    // 构建layer 控制台打印
    let console_layer = fmt::layer().with_timer(timer);

    tracing_subscriber::registry()
        .with(env_filter)
        .with(console_layer)
        .init();
}


#[tokio::main]
async fn main() {
    // 解析 .env 文件
    dotenvy::dotenv().expect("解析.env文件失败");

    let _guard = init_trace_logger();

    let cfg = AppConfig::new()
        .map_err(|e| tracing::error!("初始化配置失败：{}", e.to_string()))
        .unwrap();

    let mut opt = ConnectOptions::new(&cfg.mysql.dsn);
    opt.max_connections(cfg.mysql.max_cons);

    let app = router::init()
        .with_state(Arc::new(AppState {
            conn: Database::connect(opt).await.unwrap(),
        }));

    let listener = tokio::net::TcpListener::bind(&cfg.web.addr).await.unwrap();

    tracing::info!("服务器运行于: {}", listener.local_addr().unwrap());

    axum::serve(listener, app).with_graceful_shutdown(shutdown_signal()).await.unwrap()
}


async fn shutdown_signal() {
    let ctrl_c = async {
        signal::ctrl_c()
            .await
            .expect("failed to install Ctrl+C handler");
    };

    #[cfg(unix)]
        let terminate = async {
        signal::unix::signal(signal::unix::SignalKind::terminate())
            .expect("failed to install signal handler")
            .recv()
            .await;
    };
    #[cfg(not(unix))]
        let terminate = std::future::pending::<()>();
    tokio::select! {
        _ = ctrl_c => {},
        _ = terminate => {},
    }
    tracing::info!("signal received, starting graceful shutdown");
}